Package: lwgeom
Version: 0.2-16
Title: Bindings to Selected 'liblwgeom' Functions for Simple Features
Description: Access to selected functions found in 'liblwgeom' <https://github.com/postgis/postgis/tree/master/liblwgeom>, the light-weight geometry library used by 'PostGIS' <http://postgis.net/>.
Authors@R: c(person("Edzer", "Pebesma", role = c("aut", "cre"), email = "edzer.pebesma@uni-muenster.de", comment = c(ORCID = "0000-0001-8049-7069")),
	person("Colin", "Rundel", role = "ctb"),
	person("Andy", "Teucher", role = "ctb"),
	person("liblwgeom developers", role = "cph"))
Depends: R (>= 4.1.0)
Imports: Rcpp, units, sf (>= 1.0-15)
Suggests: covr, sp, geosphere, testthat
LinkingTo: Rcpp, sf (>= 1.0-15)
SystemRequirements: GEOS (>= 3.5.0), PROJ (>= 4.8.0), sqlite3
License: GPL-2
Copyright: file COPYRIGHTS
Encoding: UTF-8
URL: https://r-spatial.github.io/lwgeom/,
        https://github.com/r-spatial/lwgeom
BugReports: https://github.com/r-spatial/lwgeom/issues
Collate: init.R RcppExports.R geohash.R split.R subdivide.R valid.R
        transform.R bounding_circle.R bearing.R snap_to_grid.R
        startpoint.R twkb.R perimeter.R clockwise.R geod.R wkt.R
        wrap_x.R
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-03 11:42:19 UTC; edzer
Author: Edzer Pebesma [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8049-7069>),
  Colin Rundel [ctb],
  Andy Teucher [ctb],
  liblwgeom developers [cph]
Maintainer: Edzer Pebesma <edzer.pebesma@uni-muenster.de>
Repository: CRAN
Date/Publication: 2026-05-03 12:40:03 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-03 14:24:47 UTC; unix
Archs: lwgeom.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(st_as_sfc,TWKB)
S3method(st_force_polygon_cw,sf)
S3method(st_force_polygon_cw,sfc)
S3method(st_force_polygon_cw,sfg)
S3method(st_is_polygon_cw,sf)
S3method(st_is_polygon_cw,sfc)
S3method(st_is_polygon_cw,sfg)
S3method(st_linesubstring,sf)
S3method(st_linesubstring,sfc)
S3method(st_linesubstring,sfg)
S3method(st_minimum_bounding_circle,sf)
S3method(st_minimum_bounding_circle,sfc)
S3method(st_minimum_bounding_circle,sfg)
S3method(st_minimum_bounding_radius,sf)
S3method(st_minimum_bounding_radius,sfc)
S3method(st_minimum_bounding_radius,sfg)
S3method(st_snap_to_grid,sf)
S3method(st_snap_to_grid,sfc)
S3method(st_snap_to_grid,sfg)
S3method(st_split,sf)
S3method(st_split,sfc)
S3method(st_split,sfg)
S3method(st_subdivide,sf)
S3method(st_subdivide,sfc)
S3method(st_subdivide,sfg)
S3method(st_transform_proj,sf)
S3method(st_transform_proj,sfc)
S3method(st_transform_proj,sfg)
S3method(st_wrap_x,sf)
S3method(st_wrap_x,sfc)
S3method(st_wrap_x,sfg)
export(lwgeom_extSoftVersion)
export(lwgeom_make_valid)
export(st_asewkt)
export(st_astext)
export(st_endpoint)
export(st_force_polygon_cw)
export(st_geod_area)
export(st_geod_azimuth)
export(st_geod_covered_by)
export(st_geod_covers)
export(st_geod_distance)
export(st_geod_length)
export(st_geod_segmentize)
export(st_geohash)
export(st_geom_from_geohash)
export(st_is_polygon_cw)
export(st_linesubstring)
export(st_minimum_bounding_circle)
export(st_minimum_bounding_radius)
export(st_perimeter)
export(st_perimeter_2d)
export(st_perimeter_lwgeom)
export(st_snap_to_grid)
export(st_split)
export(st_startpoint)
export(st_subdivide)
export(st_transform_proj)
export(st_wrap_x)
import(sf)
importFrom(Rcpp,evalCpp)
importFrom(units,as_units)
importFrom(units,set_units)
importFrom(utils,tail)
useDynLib(lwgeom)

liblwgeom code obtained from from https://github.com/postgis/postgis
git commit d9a98c5be3c8644699220729f11ed48488548bad
configured with:
./configure --without-pgconfig --without-sfcgal --without-libiconv-prefix --without-libintl-prefix --without-json --without-protobuf --without-topology --without-raster  

removed: *_sfcgal.[ch]

all diffs, registered with "diff -c | grep -v 'Only in'", are found in file DIFFS

In addition: postgis_version.h, outcomment line 139 which defines
POSTGIS_GEOS_VERSION; ./configure now does this on the command line.

in addition: "%.*g" -> "%*g" in lwgeom_wkt.c

#' Make an invalid geometry valid
#'
#' Make an invalid geometry valid
#' @param x object of class \code{sfc}
#' @export
lwgeom_make_valid = function(x) {
	stopifnot(inherits(x, "sfc"))
	st_sfc(CPL_make_valid(x), crs = st_crs(x))
}

#' liblwgeom geodetic functions
#' 
#' liblwgeom geodetic functions for length, area, segmentizing, covers
#' @name geod
#' @param x object of class \code{sf}, \code{sfc} or \code{sfg}
#' @export
#' @details \code{st_area} will give an error message when the area spans the equator and \code{lwgeom} is linked to a proj.4 version older than 4.9.0 (see \link{lwgeom_extSoftVersion})
#' @examples
#' library(sf)
#' nc = st_read(system.file("gpkg/nc.gpkg", package="sf"))
#' st_geod_area(nc[1:3,])
#' # st_area(nc[1:3,])
st_geod_area = function(x) {
	stopifnot(st_is_longlat(x))
	p = st_crs(st_geometry(x), parameters = TRUE)
	ret = CPL_geodetic_area(st_geometry(x), p$SemiMajor, p$InvFlattening)
	units(ret) = units(p$SemiMajor^2)
	ret
}

#' @name geod
#' @export
#' @examples
#' l = st_sfc(st_linestring(rbind(c(7,52), c(8,53))), crs = 4326)
#' st_geod_length(l)
st_geod_length = function(x) {
	stopifnot(st_is_longlat(x))
	p = st_crs(st_geometry(x), parameters = TRUE)
	ret = CPL_geodetic_length(st_geometry(x), p$SemiMajor, p$InvFlattening)
	units(ret) = units(p$SemiMajor)
	ret
}

#' @name geod
#' @param max_seg_length segment length in degree, radians, or as a length unit (e.g., m)
#' @export
#' @examples
#' library(units)
#' pol = st_polygon(list(rbind(c(0,0), c(0,60), c(60,60), c(0,0))))
#' x = st_sfc(pol, crs = 4326)
#' seg = st_geod_segmentize(x[1], set_units(10, km))
#' plot(seg, graticule = TRUE, axes = TRUE)
#' @details 
#' longitude coordinates returned are rescaled to [-180,180)
st_geod_segmentize = function(x, max_seg_length) {
	stopifnot(st_is_longlat(x))
	p = st_crs(st_geometry(x), parameters = TRUE)
	if (inherits(max_seg_length, "units")) {
		tr = try(units(max_seg_length) <- as_units("rad"), silent = TRUE)
		if (inherits(tr, "try-error")) {
			units(max_seg_length) = units(p$SemiMajor) # -> m
			max_seg_length = max_seg_length / p$SemiMajor # m -> rad
		}
	} else
		stop("st_geod_segmentize needs a max_seg_length with units rad, degree, or a length unit")
	ret = st_sfc(CPL_geodetic_segmentize(st_geometry(x), max_seg_length))
	st_set_crs((ret + c(180,90)) %% 360 - c(180, 90), st_crs(x)) # rescale lon to [-180,180)
}

#' @name geod
#' @param y object of class \code{sf}, \code{sfc} or \code{sfg}
#' @param sparse logical; if \code{TRUE}, return a sparse matrix (object of class \code{sgbp}), otherwise, return a dense logical matrix.
#' @export
#' @examples
#' pole = st_polygon(list(rbind(c(0,80), c(120,80), c(240,80), c(0,80))))
#' pt = st_point(c(0,90))
#' x = st_sfc(pole, pt, crs = 4326)
#' st_geod_covers(x[c(1,1,1)], x[c(2,2,2,2)])
st_geod_covers = function(x, y, sparse = TRUE) {
	stopifnot(st_is_longlat(x))
	stopifnot(st_is_longlat(y))
	if (!all(st_dimension(x) == 2))
		stop("argument x must contain only polygons")
	if (!all(st_dimension(y) == 0))
		stop("argument y must contain only points")
	if (is.null(id <- row.names(x)))
		id = as.character(seq_along(st_geometry(x)))
	ret = structure(CPL_geodetic_covers(st_geometry(x), st_geometry(y)),
		predicate = "covers", region.id = id, ncol = length(st_geometry(y)), class = "sgbp")
	if (sparse)
		ret
	else
		as.matrix(ret)
}

#' @name geod
#' @export
st_geod_covered_by = function(x, y, sparse = TRUE) {
	ret = structure(t(st_geod_covers(y, x)), predicate = "covered_by")
	if (sparse)
		ret
	else
		as.matrix(ret)
}

#' @name geod
#' @export
#' @param tolerance double or length \code{units} value: if positive, the first distance less than \code{tolerance} is returned, rather than the true distance
#' @note this function should is used by \link[sf:geos_measures]{st_distance}, do not use it directly
#' @examples
#' pole = st_polygon(list(rbind(c(0,80), c(120,80), c(240,80), c(0,80))))
#' pt = st_point(c(30,70))
#' x = st_sfc(pole, pt, crs = 4326)
#' st_geod_distance(x, x)
st_geod_distance = function(x, y, tolerance = 0.0, sparse = FALSE) {
	stopifnot(st_is_longlat(x))
	stopifnot(st_crs(x) == st_crs(y))
	p = st_crs(st_geometry(x), parameters = TRUE)
	SemiMinor = if (is.null(p$SemiMinor)) -1.0 else p$SemiMinor
	units(tolerance) = as_units("m")
	ret = CPL_geodetic_distance(st_geometry(x), st_geometry(y), p$SemiMajor, p$InvFlattening,
		tolerance, sparse, SemiMinor)[[1]]
	if (! sparse) {
		ret[ret < 0] = NA # invalid/incalculable
		units(ret) = units(p$SemiMajor)
		ret
	} else {
		if (is.null(id <- row.names(x)))
			id = as.character(seq_along(st_geometry(x)))
		structure(ret, predicate = "st_is_within_distance", 
			region.id = id, ncol = length(st_geometry(y)), class = "sgbp")
	}
}

### Name: lw_geodetic
### Title: geodetic length, area, and predicates
### Aliases: lw_geodetic st_geod_area lw_geodetic st_geod_length
###   lw_geodetic st_geod_segmentize lw_geodetic st_geod_covers

### ** Examples

suppressPackageStartupMessages(library(sf))
suppressPackageStartupMessages(library(lwgeom))
suppressPackageStartupMessages(library(units))
nc = st_read(system.file("gpkg/nc.gpkg", package="sf"), quiet = TRUE)
st_geod_area(nc[1:3,])
# st_area(nc[1:3,])
l = st_sfc(st_linestring(rbind(c(7,52), c(8,53))), crs = 4326)
st_geod_length(l)
pol = st_polygon(list(rbind(c(0,0), c(0,60), c(60,60), c(0,0))))
x = st_sfc(pol, crs = 4326)
seg = st_geod_segmentize(x[1], set_units(10, km))
plot(seg, graticule = TRUE, axes = TRUE)
pole = st_polygon(list(rbind(c(0,80), c(120,80), c(240,80), c(0,80))))
pt = st_point(c(0,90))
x = st_sfc(pole, pt, crs = 4326)
st_geod_covers(x[c(1,1,1)], x[c(2,2,2,2)])
st_geod_covered_by(x[c(2,2)], x[c(1,1,1)])
st_geod_covers(x[c(1,1,1)], x[c(2,2,2,2)], sparse = FALSE)
st_geod_covered_by(x[c(2,2)], x[c(1,1,1)], sparse = FALSE)

# box crossing the dateline:
#box = st_polygon(list(rbind(c(179.5,0), c(179.5,1), c(-179.5,1), c(-179.5,0), c(179.5,0))))
box = st_polygon(list(rbind(c(179.5,0.1), c(179.5,1), c(-179.5,1), c(-179.5,0.1), c(179.5,0.1))))
b = st_sfc(box, crs = 4326)
units::set_units(st_geod_area(b), km^2) # approx 111^2

pt = st_point(c(30, 70))
x = st_sfc(pole, pt, pt, crs = 4326)
# st_geod_distance(x, x)
# st_geod_distance(x, x, sparse = TRUE)
st_geod_distance(x, x, tolerance = 1, sparse = TRUE)

#' Return the start and end points from lines
#' 
#' @param x line of class \code{sf}, \code{sfc} or \code{sfg}
#' @export
#' @details see \url{https://postgis.net/docs/ST_StartPoint.html} and \url{https://postgis.net/docs/ST_EndPoint.html}.
#' @return \code{sf} object representing start and end points
#' @examples
#' library(sf)
#' m = matrix(c(0, 1, 2, 0, 1, 4), ncol = 2)
#' l = st_sfc(st_linestring(m))
#' lwgeom::st_startpoint(l)
#' lwgeom::st_endpoint(l)
#' l2 = st_sfc(st_linestring(m), st_linestring(m[3:1, ]))
#' lwgeom::st_startpoint(l2)
#' lwgeom::st_endpoint(l2)
st_startpoint = function(x) {
  m <- CPL_startpoint(st_geometry(x))
  st_sfc(lapply(seq_len(nrow(m)), function(i) st_point(m[i,])), crs = st_crs(x))
}
#' @rdname st_startpoint
#' @export
st_endpoint = function(x) {
  m <- CPL_endpoint(st_geometry(x))
  st_sfc(lapply(seq_len(nrow(m)), function(i) st_point(m[i,])), crs = st_crs(x))
}

#' Splits input geometries by a vertical line and moves components falling on
#' one side of that line by a fixed amount
#' 
#' @name st_wrap_x
#' @param x object with geometries to be split
#' @param wrap x value of split line
#' @param move amount by which geometries falling to the left of the line
#'             should be translated to the right
#' @return object of the same class as \code{x}
#' @examples
#' library(sf)
#' demo(nc, ask = FALSE, echo = FALSE)
#' x = st_wrap_x(nc, -78, 10)
#' plot(x[1])
#' @export
st_wrap_x = function(x, wrap, move) UseMethod("st_wrap_x")

#' @export
st_wrap_x.sfg = function(x, wrap, move) {
	st_wrap_x(st_geometry(x), wrap, move)[[1]]
}

#' @export
st_wrap_x.sfc = function(x, wrap, move) {
    st_sfc(CPL_wrap_x(x, wrap, move), crs = st_crs(x))
}

#' @export
st_wrap_x.sf = function(x, wrap, move) {
	st_set_geometry(x, st_wrap_x(st_geometry(x), wrap, move))
}

#' Snap geometries to a grid
#' 
#' @name st_snap_to_grid
#' @param x object with geometries to be snapped
#' @param size numeric or (length) units object; grid cell size in x-, y- (and possibly z- and m-) directions
#' @param origin numeric; origin of the grid
#' @return object of the same class as \code{x}
#' @examples
#' # obtain data
#' library(sf)
#' x = st_read(system.file("gpkg/nc.gpkg", package="sf"), quiet = TRUE)[1, ] |>
#'     st_geometry() |>
#'     st_transform(3395)
#'
#' # snap to a grid of 5000 m
#' err = try(y <- st_snap_to_grid(x, 5000))
#'
#' # plot data for visual comparison
#' if (!inherits(err, "try-error")) {
#'  opar = par(mfrow = c(1, 2))
#'  plot(x, main = "orginal data")
#'  plot(y, main = "snapped to 5000 m")
#'  par(opar)
#' }
#' @export
st_snap_to_grid = function(x, size, origin) UseMethod("st_snap_to_grid")

#' @export
st_snap_to_grid.sfg = function(x, size, origin = st_point(rep(0.0,4))) {
	st_snap_to_grid(st_geometry(x), size, origin)[[1]]
}

#' @export
st_snap_to_grid.sfc = function(x, size, origin = st_point(rep(0.0,4))) {
	size = rep(as.numeric(size), length.out = 4)
	stopifnot(!isTRUE(st_is_longlat(x))) # FIXME
	units(size) = as_units("m")
	st_sfc(CPL_snap_to_grid(x, origin, size), crs = st_crs(x))
}

#' @export
st_snap_to_grid.sf = function(x, size, origin = st_point(rep(0.0,4))) {
	st_set_geometry(x, st_snap_to_grid(st_geometry(x), size, origin))
}

#' Generate the minimum bounding circle
#' @name bounding_circle
#' @param x object of class \code{sfg}, \code{sfc} or \code{sf}
#' @param nQuadSegs number of segments per quadrant (passed to \code{st_buffer})
#' @return Object of the same class as \code{x}
#' @details \code{st_minimum_bounding_circle} uses the \code{lwgeom_calculate_mbc} method also used by the PostGIS command \code{ST_MinimumBoundingCircle}. \code{st_minimum_bounding_radius} also uses the \code{lwgeom_calculate_mbc} method, but returns the centers (as a geometry) and the respective radii (as a column).
#' @examples
#' library(sf)
#'
#' x = st_multipoint(matrix(c(0,1,0,1),2,2))
#' y = st_multipoint(matrix(c(0,0,1,0,1,1),3,2))
#'
#' mbcx = st_minimum_bounding_circle(x)
#' mbcy = st_minimum_bounding_circle(y)
#'
#' if (.Platform$OS.type != "windows") {
#'   plot(mbcx, axes=TRUE); plot(x, add=TRUE)
#'   plot(mbcy, axes=TRUE); plot(y, add=TRUE)
#' }
#' 
#' nc = st_read(system.file("gpkg/nc.gpkg", package="sf"))
#' state = st_union(st_geometry(nc))
#' 
#' if (.Platform$OS.type != "windows") {
#'   plot(st_minimum_bounding_circle(state), asp=1)
#'   plot(state, add=TRUE)
#' }
#'
#' @export
st_minimum_bounding_circle = function(x, nQuadSegs = 30) UseMethod("st_minimum_bounding_circle", x)

generate_circles = function(geom, nQuadSegs = 30) {
  stopifnot(inherits(geom,"sfc"))
  
  circles = CPL_minimum_bounding_circle(geom)
  if (any(is.nan(unlist(circles))))
    stop("NaN values returned by lwgeom's lwgeom_calculate_mbc.") #nocov

  mapply(
    function(xy, r, nQuadSegs) {
      st_buffer(st_point(xy), r, nQuadSegs)
    },
    circles[["center"]], circles[["radius"]],
    MoreArgs = list(nQuadSegs = nQuadSegs),
    SIMPLIFY = FALSE
  )
}

#' @export
st_minimum_bounding_circle.sfg = function(x, nQuadSegs = 30) {
  st_minimum_bounding_circle(st_geometry(x), nQuadSegs)[[1]]
}

#' @export
st_minimum_bounding_circle.sfc = function(x, nQuadSegs = 30) {
  st_sfc(generate_circles(x, nQuadSegs), crs = st_crs(x))
}

#' @export
st_minimum_bounding_circle.sf = function(x, nQuadSegs = 30) {
  st_set_geometry(x, st_minimum_bounding_circle(st_geometry(x), nQuadSegs))
}

#' @rdname bounding_circle
#' @export
st_minimum_bounding_radius = function(x) UseMethod("st_minimum_bounding_radius", x)

generate_radii = function(geom) {
  stopifnot(inherits(geom,"sfc"))
  circles = CPL_minimum_bounding_circle(geom)
  if (any(is.nan(unlist(circles))))
    stop("NaN values returned by lwgeom's lwgeom_calculate_mbc.") #nocov
  centers = st_as_sf(st_sfc(lapply(circles[["center"]], st_point),
                            crs = st_crs(geom)))
  cbind(centers, radius = circles[["radius"]])
}

#' @export
st_minimum_bounding_radius.sfg = function(x) {
  warning("Returning only centers.")
  lapply(CPL_minimum_bounding_circle(x)[["center"]],
         st_point)
}

#' @export
st_minimum_bounding_radius.sfc = function(x) {
  warning("Returning only centers.")
  st_geometry(generate_radii(x))
}

#' @export
st_minimum_bounding_radius.sf = function(x) {
  aux = generate_radii(st_geometry(x))
  out = st_set_geometry(x, st_geometry(aux))
  cbind(out, st_drop_geometry(aux))
}

#' compute azimuth between sequence of points
#'
#' compute azimuth between sequence of points
#' @param x object of class \code{sf}, \code{sfc} or \code{sfg}
#' @param y object of class \code{sf}, \code{sfc} or \code{sfg}, or NULL
#' @export
#' @examples
#' library(sf)
#' p = st_sfc(st_point(c(7,52)), st_point(c(8,53)), crs = 4326)
#' st_geod_azimuth(p)
st_geod_azimuth = function(x, y = NULL) {
	stopifnot(st_is_longlat(x))
	stopifnot(all(st_is(x, "POINT")))
	p = st_crs(st_geometry(x), parameters = TRUE)
	if (is.null(y)) {
	  ret = CPL_geodetic_azimuth(st_geometry(x), p$SemiMajor, p$InvFlattening)
	} else {
	  ret = CPL_geodetic_azimuth(st_geometry(x),
	                             p$SemiMajor,
	                             p$InvFlattening,
	                             st_geometry(y))
	}
	units(ret) = as_units("rad")
	ret
}

#' Return a collection of geometries resulting by subdividing a geometry
#' 
#' @name st_subdivide
#' @param x object with geometries to be subdivided
#' @param max_vertices integer; maximum size of the subgeometries (at least 8)
#' @return object of the same class as \code{x}
#' @examples
#' library(sf)
#' demo(nc, ask = FALSE, echo = FALSE)
#' x = st_subdivide(nc, 10)
#' plot(x[1])
#' @export
st_subdivide = function(x, max_vertices) UseMethod("st_subdivide")

#' @export
st_subdivide.sfg = function(x, max_vertices = 256) {
	st_subdivide(st_geometry(x), max_vertices)[[1]]
}

#' @export
st_subdivide.sfc = function(x, max_vertices = 256) {
    st_sfc(CPL_subdivide(x, max_vertices), crs = st_crs(x))
}

#' @export
st_subdivide.sf = function(x, max_vertices = 256) {
	st_set_geometry(x, st_subdivide(st_geometry(x), max_vertices))
}

#' Return Well-known Text representation of simple feature geometry
#'
#' Return Well-known Text representation of simple feature geometry or coordinate reference system
#' @param x object of class `sfg`, `sfc`, or `sf`
#' @param digits integer; number of decimal digits to print
#' @param ... ignored
#' @param EWKT logical; use PostGIS Enhanced WKT (includes srid)
#' @name st_astext
#' @details The returned WKT representation of simple feature geometry conforms to the
#' [simple features access](https://www.ogc.org/standards/sfa/) specification and extensions (if `EWKT = TRUE`),
#' [known as EWKT](http://postgis.net/docs/using_postgis_dbmanagement.html#EWKB_EWKT), supported by
#' PostGIS and other simple features implementations for addition of SRID to a WKT string.
#' @md
#' @examples 
#' library(sf)
#' pt <- st_sfc(st_point(c(1.0002,2.3030303)), crs = 4326)
#' st_astext(pt, 3)
#' st_asewkt(pt, 3)
#' @export
st_astext <- function(x, digits = getOption("digits"), ..., EWKT = FALSE) {
  if (! EWKT && !inherits(x, "sfg"))
    st_crs(x) <- NA_crs_
  CPL_sfc_to_wkt(st_geometry(x), as.integer(digits))
}

#' @name st_astext
#' @inheritParams st_astext
#' @details `st_asewkt()` returns the Well-Known Text (WKT) representation of 
#' the geometry with SRID meta data.
#' @md
#' @export
st_asewkt <- function(x, digits = options("digits")) {
  st_astext(x, digits = digits, EWKT = TRUE)
}

context("test-as_text.R")
library(sf)

test_that("Prints Points", {
  pt <- st_sfc(st_point(c(1.0002,2.3030303)), crs = 4326)
  expect_equal(st_asewkt(pt, 1), "SRID=4326;POINT(1 2.3)")
  expect_equal(st_astext(pt, 2, EWKT = FALSE), "POINT(1 2.3)")
  expect_equal(st_astext(pt, 3, EWKT = FALSE), "POINT(1 2.303)")
  expect_equal(st_astext(pt, 10, EWKT = FALSE), "POINT(1.0002 2.3030303)")
})

test_that("Prints Polygons and Lines", {
  pol <- st_sfc(st_polygon(list(
    rbind(c(0,0),c(0.5,0),c(0.5,0.5),c(0.5,0),c(1,0),c(1,1),c(0,1),c(0,0))
    )))
  txt <- "POLYGON((0 0,0.5 0,0.5 0.5,0.5 0,1 0,1 1,0 1,0 0))"
  expect_equal(st_astext(pol), txt)
  ln <- st_cast(pol, "LINESTRING")
  txt <- "LINESTRING(0 0,0.5 0,0.5 0.5,0.5 0,1 0,1 1,0 1,0 0)"
  expect_equal(st_astext(ln), txt)
})

suppressPackageStartupMessages(library(sf))
library(sp)
suppressPackageStartupMessages(library(units))
library(geosphere)

x = st_sfc(
st_point(c(0,0)),
st_point(c(1,0)),
st_point(c(2,0)),
st_point(c(3,0)),
crs = 4326
)

y = st_sfc(
st_point(c(0,10)),
st_point(c(1,0)),
st_point(c(2,0)),
st_point(c(3,0)),
st_point(c(4,0)),
crs = 4326
)

st_crs(y) = 4326
st_crs(x) = 4326
sf_use_s2(FALSE)
(d.sf = st_distance(x, y))

#' Return a collection of geometries resulting by splitting a geometry
#' 
#' @name st_split
#' @param x object with geometries to be splitted
#' @param y object split with (blade); if \code{y} contains more than one feature geometry, the geometries are \link[sf:geos_combine]{st_combine} 'd
#' @return object of the same class as \code{x}
#' @examples
#' library(sf)
#' l = st_as_sfc('MULTILINESTRING((10 10, 190 190), (15 15, 30 30, 100 90))')
#' pt = st_sfc(st_point(c(30,30)))
#' st_split(l, pt)
#' @export
st_split = function(x, y) UseMethod("st_split")

#' @export
st_split.sfg = function(x, y) {
	st_split(st_geometry(x), st_geometry(y))[[1]]
}

#' @export
st_split.sfc = function(x, y) {
  y = st_geometry(y)
    if (length(y) > 1) y = sf::st_combine(y)
	if (inherits(x, "sfc_POLYGON") || inherits(x, "sfc_MULTIPOLYGON"))
    	stopifnot(inherits(y, "sfc_LINESTRING") || inherits(y, "sfc_MULTILINESTRING"))
	else
		stopifnot(inherits(x, "sfc_LINESTRING") || inherits(x, "sfc_MULTILINESTRING"))
    st_sfc(CPL_split(x, y), crs = st_crs(x))
}

#' @export
st_split.sf = function(x, y) {
	st_set_geometry(x, st_split(st_geometry(x), y))
}

#' get substring from linestring
#' @export
#' @param x object of class \code{sfc}, \code{sf} or \code{sfg}
#' @param from relative distance from origin (in [0,1])
#' @param to relative distance from origin (in [0,1])
#' @param ... ignored
#' @param tolerance tolerance parameter, when to snap to line node node
#' @return object of class \code{sfc}
#' @examples
#' library(sf)
#' lines = st_sfc(st_linestring(rbind(c(0,0), c(1,2), c(2,0))), crs = 4326)
#' spl = st_linesubstring(lines, 0.2, 0.8) # should warn
#' plot(st_geometry(lines), col = 'red', lwd = 3)
#' plot(spl, col = 'black', lwd = 3, add = TRUE)
#' st_linesubstring(lines, 0.49999, 0.8) # three points
#' st_linesubstring(lines, 0.49999, 0.8, 0.001) # two points: snap start to second node
st_linesubstring = function(x, from, to, tolerance, ...) UseMethod("st_linesubstring")

#' @export
st_linesubstring.sfc = function(x, from, to, tolerance = 0.0, ...) {
	if (isTRUE(st_is_longlat(x)))
		warning("st_linesubstring does not follow a geodesic; you may want to use st_geod_segmentize first")
	st_sfc(CPL_linesubstring(x, from, to, tolerance), crs = st_crs(x))
}

#' @export
st_linesubstring.sf = function(x, from, to, tolerance = 0.0, ...) {
	if (isTRUE(st_is_longlat(x)))
		warning("st_linesubstring does not follow a geodesic; you may want to use st_geod_segmentize first")
	st_set_geometry(x, st_linesubstring(st_geometry(x), from, to, tolerance))
}

#' @export
st_linesubstring.sfg = function(x, from, to, tolerance = 0.0, ...) {
	CPL_linesubstring(st_geometry(x), from, to, tolerance)[[1]]
}

context("lwgeom")

test_that("st_make_valid works", {
	library(sf)
	x = st_sfc(st_polygon(list(rbind(c(0,0),c(0.5,0),c(0.5,0.5),c(0.5,0),c(1,0),c(1,1),c(0,1),c(0,0)))))
	fls = suppressWarnings(sf::st_is_valid(x, FALSE))
	expect_false(fls)

   	y = st_make_valid(x)
	expect_true(st_is_valid(y))
   	expect_true(st_is_valid(st_make_valid(x[[1]])))
   	expect_true(st_is_valid(lwgeom_make_valid(x)))
   	expect_true(st_is_valid(st_make_valid(st_sf(a = 1, geom = x))))
	expect_equal(lwgeom::st_geohash(st_sfc(st_point(c(1.5,3.5)), st_point(c(0,90))), 2), c( "s0","up"))
	expect_equal(lwgeom::st_geohash(st_sfc(st_point(c(1.5,3.5)), st_point(c(0,90))), 10),
		c("s095fjhkbx","upbpbpbpbp"))
	l = st_as_sfc('MULTILINESTRING((10 10, 190 190), (15 15, 30 30, 100 90))')
	pt = st_sfc(st_point(c(30,30)))
	expect_silent(lwgeom::st_split(l, pt)) # sfc
	expect_silent(lwgeom::st_split(l[[1]], pt)) # sfg
	expect_silent(lwgeom::st_split(st_sf(a = 1, geom = l), pt)) # sf
	# https://github.com/r-spatial/sf/issues/509 :
	p1 = st_point(c(7,52))
	geom.sf = st_sfc(p1, crs = 4326)
	x <- st_transform_proj(geom.sf, "+proj=wintri")
	if (sf_extSoftVersion()["proj.4"] >= "6.0.0") {
		x2 <- st_transform_proj(geom.sf, c("EPSG:4326", "+proj=wintri"))
	}
	x3 <- st_transform_proj(geom.sf, st_crs(3857))
	p = st_crs(4326)$proj4string
	x <- st_transform_proj(structure(geom.sf[[1]], proj4string = p), "+proj=wintri")
	nc = st_read(system.file("gpkg/nc.gpkg", package="sf"), quiet = TRUE)
	st_transform_proj(nc[1,], "+proj=wintri +over")
	lwgeom_extSoftVersion()
})

#test_that("st_minimum_bounding_circle works", {
#  library(sf)
#  x = st_multipoint(matrix(c(0,1,0,1),2,2))
#  y = st_multipoint(matrix(c(0,0,1,0,1,1),3,2))
#  plot(st_minimum_bounding_circle(x), axes=TRUE); plot(x, add=TRUE)
#  plot(st_minimum_bounding_circle(y), axes=TRUE); plot(y, add=TRUE)
#  nc = st_read(system.file("shape/nc.shp", package="sf"), quiet = TRUE)
#  state = st_union(st_geometry(nc))
#  st_minimum_bounding_circle(state)
#  st_minimum_bounding_circle(st_sf(st = "nc", geom = state))
#})

test_that("st_subdivide works", {
	library(sf)
	x = st_read(system.file("gpkg/nc.gpkg", package="sf"), quiet = TRUE)
	expect_silent(st_subdivide(x, 10))
	expect_silent(st_subdivide(st_geometry(x), 10))
	expect_silent(st_subdivide(st_geometry(x)[[1]], 10))
})

test_that("st_snap_to_grid_works", {
	# make data
	library(sf)
	x = st_read(system.file("gpkg/nc.gpkg", package="sf"), quiet = TRUE) %>%
			st_transform(3395)
	# snap to grid
	err <- try(y1 <- st_snap_to_grid(x, 5000), silent = TRUE)
	if (inherits(err, "try-error")) # not available in liblwgeom < 2.5.0
		skip("snap_to_grid not available in this liblwgeom version")
	y2 = st_snap_to_grid(st_geometry(x), 5000)
	y3 = st_snap_to_grid(st_geometry(x)[[1]], 5000)
	# check that output class match inputs
	expect_is(y1, "sf")
	expect_is(y2, "sfc")
	expect_is(y3, "sfg")
	# check that outputs contain correct number of geometries
	expect_equal(nrow(x), nrow(y1))
	expect_equal(nrow(x), length(y2))
	# check that outputs have correct crs
	expect_equal(st_crs(x), st_crs(y1))
	expect_equal(st_crs(x), st_crs(y2))
	# check that outputs have been snapped correctly, 
	# i.e. the coordinates of the geometries divided by the tolerance (5000)
	# should not yield a remainder
	y1_m <- do.call(rbind, lapply(st_cast(st_geometry(y1), "MULTIPOINT"), as.matrix))
	expect_true(all(c(y1_m %% 5000) == 0))
	y2_m <- do.call(rbind, lapply(st_cast(y2, "MULTIPOINT"), as.matrix))
	expect_true(all(c(y2_m %% 5000) == 0))
	expect_true(all(c(as.matrix(st_cast(y3, "MULTIPOINT")) %% 5000) == 0))
})

#test_that("st_transform_proj finds sf's PROJ files", {
#  # skip_on_os("mac") # FIXME: in sf rather than here
#  library(sf)
#  nc <- st_read(system.file("gpkg/nc.gpkg", package="sf"), quiet = TRUE)
#  bb1 = st_bbox(nc)
#  bb2 = st_bbox(st_transform(nc, "+proj=longlat"))
#  bb3 = st_bbox(st_transform_proj(nc, "+proj=longlat"))
#  bb4 = st_bbox(st_transform_proj(nc, st_crs(4326)$proj4string))
#  # expect_false(any(bb1 == bb2))
#  # expect_true(all.equal(as.numeric(bb2), as.numeric(bb3)))
#  # expect_true(all.equal(as.numeric(bb4), as.numeric(bb3)))
#})

test_that("st_linesubstring warns on 4326", {
  library(sf)
  lines = st_sfc(st_linestring(rbind(c(0,0), c(1,2), c(2,0))), crs = 4326)
  library(lwgeom)
  expect_warning(spl <- st_linesubstring(lines, 0.2, 0.8))
  expect_silent(spl <- st_linesubstring(lines[[1]], 0.2, 0.8)) # sfg has no crs
  expect_warning(spl <- st_linesubstring(st_sf(lines), 0.2, 0.8))
  plot(st_geometry(lines), col = 'red', lwd = 3)
  plot(spl, col = 'black', lwd = 3, add = TRUE)
})

#test_that("st_startpoint works", {
#  library(sf)
#  library(lwgeom)
#  sp = st_startpoint(st_sfc(st_linestring(matrix(1:10,,2)), st_linestring(matrix(3:12,,2)),crs=4326))
#})

test_that("st_wrap_x works", {
  library(sf)
  library(lwgeom)
  
  nc <- st_read(system.file("gpkg/nc.gpkg", package="sf"), quiet = TRUE)
  
  splitline <- -78
  offset <- 10
  
  x <- st_wrap_x(nc, splitline, offset)
  
  expect_equal(nrow(x), nrow(nc))
  expect_equal(st_bbox(x)$xmin, splitline, check.attributes = FALSE)
  expect_equal(st_bbox(x)$xmax, splitline + offset, check.attributes = FALSE)
})

test_that("st_split works", {
  library(lwgeom)
  library(sf)
  
  l = st_as_sfc('MULTILINESTRING((10 10, 190 10, 190 190), (15 15, 30 30, 100 90))')
  
  pts = st_sfc(st_multipoint(matrix(c(30, 30,
                                      190, 10),
                                    ncol = 2,
                                    byrow = TRUE)))
  pts = st_cast(pts, "POINT")
  pts_sf = st_sf(pts)
  
  splitted1 = st_split(l, pts)
  splitted1 = st_collection_extract(splitted1, "LINESTRING")
  
  splitted2 = st_split(l, pts_sf)
  splitted2 = st_collection_extract(splitted2, "LINESTRING")
  
  expect_equal(length(splitted1), 4)
  expect_equal(splitted1, splitted2)
})


test_that('st_geod_azimuth works', {
  library(lwgeom)
  library(sf)
  library(units)
  
  p = st_sfc(st_point(c(0,0)), st_point(c(0,0)), crs = 4326)
  
  p_x = st_sfc(st_point(c(0,0)), st_point(c(10,0)), crs = 4326)
  
  p_y = st_sfc(st_point(c(0,0)), st_point(c(0,10)), crs = 4326)
  
  expect_equal(st_geod_azimuth(p), as_units(NaN, 'rad'))
  expect_equal(st_geod_azimuth(p_x), as_units(1.57, 'rad'), tolerance = 42)
  expect_equal(st_geod_azimuth(p_y), as_units(0, 'rad'))
  
  # Check pairwise azimuth
  expect_equal(nrow(st_geod_azimuth(p, y = p)), nrow(p))
  
  expect_equal(st_geod_azimuth(p, y = p), rep(as_units(NaN, 'rad'), 2))
  expect_equal(st_geod_azimuth(p, y = p_x), as_units(c(NaN, 1.57), 'rad'), 
               tolerance = 42)
  expect_equal(st_geod_azimuth(p, y = p_y), as_units(c(NaN, 0), 'rad'))
})

#' Compute perimeter from polygons or other geometries
#' 
#' @name perimeter
#' @param x object of class \code{sf}, \code{sfc} or \code{sfg}
#' @return numerical vector with perimeter for each feature (geometry), with unit of measure when possible
#' @export
st_perimeter_lwgeom = function(x) {
	if (isTRUE(st_is_longlat(x)))
		stop("for perimeter of longlat geometry, cast to LINESTRING and use st_length") # nocov
	ret = CPL_perimeter(st_geometry(x), FALSE)
	units(ret) = st_crs(x, parameters=TRUE)$ud_unit
	ret
}

#' compute perimeter from polygons or other geometries
#' 
#' Deprecated.
#' Use \code{sf::st_perimeter()} or \code{st_perimeter_lwgeom()} instead.
#' @keywords internal
#' @name perimeter-deprecated
#' @param x object of class \code{sf}, \code{sfc} or \code{sfg}
#' @export
st_perimeter = function(x) {
  .Deprecated("sf::st_perimeter or lwgeom::st_perimeter_lwgeom")
  # for back compatibility 
  st_perimeter_lwgeom(x)
}

#' @export
#' @rdname perimeter
st_perimeter_2d = function(x) {
	if (isTRUE(st_is_longlat(x)))
		stop("for perimeter of longlat geometry, cast to LINESTRING and use st_length") # nocov
	ret = CPL_perimeter(st_geometry(x), TRUE)
	units(ret) = st_crs(x, parameters=TRUE)$ud_unit
	ret
}

#' Transform or convert coordinates of simple features directly with Proj.4 (bypassing GDAL)
#'
#' Transform or convert coordinates of simple features directly with Proj.4 (bypassing GDAL)
#'
#' @param x object of class sf, sfc or sfg
#' @param crs character; target CRS, or, in case of a length 2 character vector, source and target CRS
#' @param ... ignored
#' @details Transforms coordinates of object to new projection, using PROJ directly rather than the GDAL API used by \link[sf]{st_transform}.
#' 
#' If \code{crs} is a single CRS, it forms the target CRS, and in that case the source CRS is obtained as \code{st_crs(x)}. Since this presumes that the source CRS is accepted by GDAL (which is not always the case), a second option is to specify the source and target CRS as two proj4strings in argument \code{crs}. In the latter case, \code{st_crs(x)} is ignored and may well be \code{NA}.
#' @examples
#' library(sf)
#' p1 = st_point(c(7,52))
#' p2 = st_point(c(-30,20))
#' sfc = st_sfc(p1, p2, crs = 4326)
#' sfc
#' st_transform_proj(sfc, "+proj=wintri")
#' @export
st_transform_proj = function(x, crs, ...) UseMethod("st_transform_proj")

#' @name st_transform_proj
#' @export
st_transform_proj.sfc = function(x, crs, ...) {
	if (inherits(crs, "crs"))
		crs = if (sf_extSoftVersion()["proj.4"] >= "6.0.0")
				crs$wkt
			else
				crs$proj4string
	if (length(crs) == 1) # only target CRS given: get source crs
		crs = if (sf_extSoftVersion()["proj.4"] >= "6.0.0")
				c(st_crs(x)$wkt, crs) # c(input, output)
			else
				c(st_crs(x)$proj4string, crs) # c(input, output)
	stopifnot(is.character(crs), length(crs) == 2)
	ret = CPL_lwgeom_transform(x, crs)
	ret = try(st_sfc(ret, crs = crs[2]))
	if (inherits(ret, "try-error")) {
		warning("CRS of returned object is NA, since the target CRS is not recognized by GDAL")
		st_sfc(ret, crs = NA_crs_)
	} else
		ret
}

#' @name st_transform_proj
#' @export
#' @examples
#' library(sf)
#' nc = st_read(system.file("shape/nc.shp", package="sf"))
#' st_transform_proj(nc[1,], "+proj=wintri +over")
st_transform_proj.sf = function(x, crs, ...) {
	x[[ attr(x, "sf_column") ]] = st_transform_proj(st_geometry(x), crs, ...)
	x
}

#' @name st_transform_proj
#' @export
#' @details The \code{st_transform_proj} method for \code{sfg} objects assumes that the CRS of the object is available as an attribute of that name.
#' @examples
#' st_transform_proj(structure(p1, proj4string = "+init=epsg:4326"), "+init=epsg:3857")
st_transform_proj.sfg = function(x, crs, ...) {
	if (missing(crs))
		stop("argument crs cannot be missing") # nocov
	if (length(crs) == 1) {
		if (is.null(attr(x, "proj4string")))
			stop("x does not have a proj4string attribute") # nocov
		if (!is.character(attr(x, "proj4string")))
			stop("proj4string attribute should be a character string") # nocov
		crs = c(attr(x, "proj4string"), crs)
	}
	structure(st_transform_proj(st_sfc(x), crs, ...)[[1]], proj4string = tail(crs, 1))
}

#' Force a POLYGON or MULTIPOLYGON to be clockwise
#' 
#' Check if a POLYGON or MULTIPOLYGON is clockwise, and if not make it so. 
#' According to the 'Right-hand-rule', outer rings should be clockwise, and 
#' inner holes should be counter-clockwise
#' 
#' 
#' @name st_force_polygon_cw
#' @param x object with polygon geometries
#' @return object of the same class as \code{x}
#' @export
#' 
#' @examples 
#' library(sf)
#' polys <- st_sf(cw = c(FALSE, TRUE), 
#'                st_as_sfc(c('POLYGON ((0 0, 1 0, 1 1, 0 0))', 
#'                            'POLYGON ((1 1, 2 2, 2 1, 1 1))')))
#'
#' st_force_polygon_cw(polys)
#' st_force_polygon_cw(st_geometry(polys))
#' st_force_polygon_cw(st_geometry(polys)[[1]])
st_force_polygon_cw = function(x) UseMethod("st_force_polygon_cw")

#' @export
st_force_polygon_cw.sfg = function(x) {
	st_force_polygon_cw(st_geometry(x))[[1]]
}

#' @export
st_force_polygon_cw.sfc = function(x) {
  st_sfc(CPL_force_polygon_cw(x), crs = st_crs(x))
}

#' @export
st_force_polygon_cw.sf = function(x) {
	st_set_geometry(x, st_force_polygon_cw(st_geometry(x)))
}

#' Check if a POLYGON or MULTIPOLYGON is clockwise
#' 
#' Check if a POLYGON or MULTIPOLYGON is clockwise. According to the 
#' 'Right-hand-rule', outer rings should be clockwise, and inner holes
#' should be counter-clockwise
#' 
#' @name st_is_polygon_cw
#' @param x object with polygon geometries
#' @return logical with length the same number of features in `x`
#' @export
#' 
#' @examples 
#' library(sf)
#' polys <- st_sf(cw = c(FALSE, TRUE), 
#'                st_as_sfc(c('POLYGON ((0 0, 1 0, 1 1, 0 0))', 
#'                            'POLYGON ((1 1, 2 2, 2 1, 1 1))')))
#' 
#' st_is_polygon_cw(polys)
#' st_is_polygon_cw(st_geometry(polys))
#' st_is_polygon_cw(st_geometry(polys)[[1]])
st_is_polygon_cw = function(x) UseMethod("st_is_polygon_cw")

#' @export
st_is_polygon_cw.sfg = function(x) {
  st_is_polygon_cw(st_geometry(x))[[1]]
}

#' @export
st_is_polygon_cw.sfc = function(x) {
  CPL_is_polygon_cw(x)
}

#' @export
st_is_polygon_cw.sf = function(x) {
  st_is_polygon_cw(st_geometry(x))
}

context("test-clockwise.R")

library(sf)

polys <- st_sf(cw = c(FALSE, TRUE), 
               st_as_sfc(c('POLYGON ((0 0, 1 0, 1 1, 0 0))', 
                           'POLYGON ((1 1, 2 2, 2 1, 1 1))')))

test_that("st_is_polygon_cw works on all classes", {
  expect_equal(c(FALSE, TRUE), st_is_polygon_cw(polys))
  expect_equal(c(FALSE, TRUE), st_is_polygon_cw(st_geometry(polys)))
  expect_false(st_is_polygon_cw(st_geometry(polys)[[1]]))
})

test_that("st_force_polygon_cw works on all classes", {
  
  out_sf <- st_force_polygon_cw(polys)
  out_sfc <- st_force_polygon_cw(st_geometry(polys))
  out_sfg <- st_force_polygon_cw(st_geometry(polys)[[1]])
  
  expect_true(all(st_is_polygon_cw(out_sf)))
  expect_true(all(st_is_polygon_cw(out_sfc)))
  expect_true(all(st_is_polygon_cw(out_sfg)))
})

test_that("st_force_polygon_cw preserves crs", {
  st_crs(polys) <- 4326
  expect_equal(st_crs(st_force_polygon_cw(polys)), 
               st_crs(polys))
  expect_equal(st_crs(st_force_polygon_cw(st_geometry(polys))), 
               st_crs(polys))
})

test_that("st_force_polyfon_cw works with Single polygon, ccw exterior ring only", {
  obj <- st_as_sfc('POLYGON ((0 0, 1 0, 1 1, 0 0))')
  expect_false(st_is_polygon_cw(obj))
  ret <- st_force_polygon_cw(obj)
  expect_true(all(st_is_polygon_cw(ret)))
})

test_that("st_force_polyfon_cw works with Single polygon, cw exterior ring only", {
  obj <- st_as_sfc('POLYGON ((0 0, 1 1, 1 0, 0 0))')
  expect_true(st_is_polygon_cw(obj))
  ret <- st_force_polygon_cw(obj)
  expect_true(all(st_is_polygon_cw(ret)))
})

test_that("st_force_polyfon_cw works with Single polygon, ccw exterior ring, cw interior rings", {
  obj <- st_as_sfc('POLYGON ((0 0, 10 0, 10 10, 0 10, 0 0), (1 1, 1 2, 2 2, 1 1), (5 5, 5 7, 7 7, 5 5))')
  expect_false(st_is_polygon_cw(obj))
  ret <- st_force_polygon_cw(obj)
  expect_true(all(st_is_polygon_cw(ret)))
})

test_that("st_force_polyfon_cw works with Single polygon, cw exterior ring, ccw interior rings", {
  obj <- st_as_sfc('POLYGON ((0 0, 0 10, 10 10, 10 0, 0 0), (1 1, 2 2, 1 2, 1 1), (5 5, 7 7, 5 7, 5 5))')
  expect_true(st_is_polygon_cw(obj))
  ret <- st_force_polygon_cw(obj)
  expect_true(all(st_is_polygon_cw(ret)))
})

test_that("st_force_polyfon_cw works with Single polygon, ccw exterior ring, mixed interior rings", {
  obj <- st_as_sfc('POLYGON ((0 0, 10 0, 10 10, 0 10, 0 0), (1 1, 1 2, 2 2, 1 1), (5 5, 7 7, 5 7, 5 5))')
  expect_false(st_is_polygon_cw(obj))
  ret <- st_force_polygon_cw(obj)
  expect_true(all(st_is_polygon_cw(ret)))
})

test_that("st_force_polyfon_cw works with Single polygon, cw exterior ring, mixed interior rings", {
  obj <- st_as_sfc('POLYGON ((0 0, 0 10, 10 10, 10 0, 0 0), (1 1, 2 2, 1 2, 1 1), (5 5, 5 7, 7 7, 5 5))')
  expect_false(st_is_polygon_cw(obj))
  ret <- st_force_polygon_cw(obj)
  expect_true(all(st_is_polygon_cw(ret)))
})

test_that("st_force_polyfon_cw works with MultiPolygon, ccw exterior rings only", {
  obj <- st_as_sfc('MULTIPOLYGON (((0 0, 1 0, 1 1, 0 0)), ((100 0, 101 0, 101 1, 100 0)))')
  expect_false(st_is_polygon_cw(obj))
  ret <- st_force_polygon_cw(obj)
  expect_true(all(st_is_polygon_cw(ret)))
})

test_that("st_force_polyfon_cw works with MultiPolygon, cw exterior rings only", {
  obj <- st_as_sfc('MULTIPOLYGON (((0 0, 1 1, 1 0, 0 0)), ((100 0, 101 1, 101 0, 100 0)))')
  expect_true(st_is_polygon_cw(obj))
  ret <- st_force_polygon_cw(obj)
  expect_true(all(st_is_polygon_cw(ret)))
})

test_that("st_force_polyfon_cw works with MultiPolygon, mixed exterior rings", {
  obj <- st_as_sfc('MULTIPOLYGON (((0 0, 1 0, 1 1, 0 0)), ((100 0, 101 1, 101 0, 100 0)))')
  expect_false(st_is_polygon_cw(obj))
  ret <- st_force_polygon_cw(obj)
  expect_true(all(st_is_polygon_cw(ret)))
})

#' compute geohash from (average) coordinates
#' 
#' @param x object of class \code{sf}, \code{sfc} or \code{sfg}
#' @param precision integer; precision (length) of geohash returned. From the liblwgeom source: ``where the precision is non-positive, a precision based on the bounds of the feature. Big features have loose precision. Small features have tight precision.''
#' @export
#' @details see \url{https://en.wikipedia.org/wiki/Geohash}.
#' @return `st_geohash` returns a character vector with geohashes; empty or full geometries result in `NA`
#' 
#' `st_geom_from_geohash` returns a (bounding box) polygon for each geohash if `raw` is `FALSE`, if `raw` is `TRUE` a matrix is returned with bounding box coordinates on each row.
#' @examples
#' library(sf)
#' lwgeom::st_geohash(st_sfc(st_point(c(1.5,3.5)), st_point(c(0,90))), 2)
#' lwgeom::st_geohash(st_sfc(st_point(c(1.5,3.5)), st_point(c(0,90))), 10)
st_geohash = function(x, precision = 0) {
	x = st_geometry(x)
	ret = vector("character", length(x))
	is_na = st_is_empty(x) | st_is_full(x)
	ret[is_na] = NA_character_
	ret[!is_na] = CPL_geohash(x[!is_na], precision)
	ret
}

#' @export
#' @name st_geohash
#' @param hash character vector with geohashes
#' @param raw logical; if `TRUE`, return a matrix with bounding box coordinates on each row
#' @param crs object of class `crs`
#' @examples
#' st_geom_from_geohash(c('9qqj7nmxncgyy4d0dbxqz0', 'up'), raw = TRUE)
#' o = options(digits = 20) # for printing purposes
#' st_geom_from_geohash(c('9qqj7nmxncgyy4d0dbxqz0', 'u1hzz631zyd63zwsd7zt'))
#' st_geom_from_geohash('9qqj7nmxncgyy4d0dbxqz0', 4) 
#' st_geom_from_geohash('9qqj7nmxncgyy4d0dbxqz0', 10)
#' options(o)
st_geom_from_geohash = function(hash, precision = -1, crs = st_crs('OGC:CRS84'), raw = FALSE) {
	stopifnot(is.character(hash), is.numeric(precision), length(precision) == 1)
	m = CPL_bbox_from_geohash(hash, as.integer(precision))
	bb = matrix(m, nrow = 4)
	rownames(bb) = c("xmin", "ymin", "xmax", "ymax")
	if (raw)
		t(bb)
	else { 
		bb = apply(bb, 2, sf::st_bbox, simplify = FALSE)
		sf::st_set_crs(do.call(c, lapply(bb, sf::st_as_sfc)), crs)
	}
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

CPL_geodetic_area <- function(sfc, semi_major, inv_flattening) {
    .Call('_lwgeom_CPL_geodetic_area', PACKAGE = 'lwgeom', sfc, semi_major, inv_flattening)
}

CPL_geodetic_length <- function(sfc, semi_major, inv_flattening) {
    .Call('_lwgeom_CPL_geodetic_length', PACKAGE = 'lwgeom', sfc, semi_major, inv_flattening)
}

CPL_geodetic_azimuth <- function(sfc, semi_major, inv_flattening, sfc2_ = NULL) {
    .Call('_lwgeom_CPL_geodetic_azimuth', PACKAGE = 'lwgeom', sfc, semi_major, inv_flattening, sfc2_)
}

CPL_geodetic_segmentize <- function(sfc, max_seg_length) {
    .Call('_lwgeom_CPL_geodetic_segmentize', PACKAGE = 'lwgeom', sfc, max_seg_length)
}

CPL_geodetic_covers <- function(sfc1, sfc2) {
    .Call('_lwgeom_CPL_geodetic_covers', PACKAGE = 'lwgeom', sfc1, sfc2)
}

CPL_geodetic_distance <- function(sfc1, sfc2, semi_major, inv_flattening, tolerance, sparse, semi_minor = -1.0) {
    .Call('_lwgeom_CPL_geodetic_distance', PACKAGE = 'lwgeom', sfc1, sfc2, semi_major, inv_flattening, tolerance, sparse, semi_minor)
}

CPL_geos_version <- function(b = FALSE) {
    .Call('_lwgeom_CPL_geos_version', PACKAGE = 'lwgeom', b)
}

CPL_init_lwgeom <- function(l) {
    .Call('_lwgeom_CPL_init_lwgeom', PACKAGE = 'lwgeom', l)
}

CPL_lwgeom_version <- function(b = FALSE) {
    .Call('_lwgeom_CPL_lwgeom_version', PACKAGE = 'lwgeom', b)
}

CPL_sfc_from_twkb <- function(twkb) {
    .Call('_lwgeom_CPL_sfc_from_twkb', PACKAGE = 'lwgeom', twkb)
}

CPL_make_valid <- function(sfc) {
    .Call('_lwgeom_CPL_make_valid', PACKAGE = 'lwgeom', sfc)
}

CPL_split <- function(sfc, blade) {
    .Call('_lwgeom_CPL_split', PACKAGE = 'lwgeom', sfc, blade)
}

CPL_wrap_x <- function(sfc, wrap, move) {
    .Call('_lwgeom_CPL_wrap_x', PACKAGE = 'lwgeom', sfc, wrap, move)
}

CPL_geohash <- function(sfc, prec) {
    .Call('_lwgeom_CPL_geohash', PACKAGE = 'lwgeom', sfc, prec)
}

CPL_bbox_from_geohash <- function(h, prec) {
    .Call('_lwgeom_CPL_bbox_from_geohash', PACKAGE = 'lwgeom', h, prec)
}

CPL_lwgeom_transform <- function(sfc, p4s) {
    .Call('_lwgeom_CPL_lwgeom_transform', PACKAGE = 'lwgeom', sfc, p4s)
}

CPL_minimum_bounding_circle <- function(sfc) {
    .Call('_lwgeom_CPL_minimum_bounding_circle', PACKAGE = 'lwgeom', sfc)
}

CPL_subdivide <- function(sfc, max_vertices = 256L) {
    .Call('_lwgeom_CPL_subdivide', PACKAGE = 'lwgeom', sfc, max_vertices)
}

CPL_snap_to_grid <- function(sfc, origin, size) {
    .Call('_lwgeom_CPL_snap_to_grid', PACKAGE = 'lwgeom', sfc, origin, size)
}

CPL_perimeter <- function(sfc, do2d = FALSE) {
    .Call('_lwgeom_CPL_perimeter', PACKAGE = 'lwgeom', sfc, do2d)
}

CPL_is_polygon_cw <- function(sfc) {
    .Call('_lwgeom_CPL_is_polygon_cw', PACKAGE = 'lwgeom', sfc)
}

CPL_force_polygon_cw <- function(sfc) {
    .Call('_lwgeom_CPL_force_polygon_cw', PACKAGE = 'lwgeom', sfc)
}

CPL_startpoint <- function(sfc) {
    .Call('_lwgeom_CPL_startpoint', PACKAGE = 'lwgeom', sfc)
}

CPL_endpoint <- function(sfc) {
    .Call('_lwgeom_CPL_endpoint', PACKAGE = 'lwgeom', sfc)
}

CPL_sfc_to_wkt <- function(sfc, precision) {
    .Call('_lwgeom_CPL_sfc_to_wkt', PACKAGE = 'lwgeom', sfc, precision)
}

CPL_proj_version <- function(b = FALSE) {
    .Call('_lwgeom_CPL_proj_version', PACKAGE = 'lwgeom', b)
}

CPL_use_proj4_init_rules <- function(v) {
    .Call('_lwgeom_CPL_use_proj4_init_rules', PACKAGE = 'lwgeom', v)
}

CPL_set_data_dir <- function(data_dir) {
    .Call('_lwgeom_CPL_set_data_dir', PACKAGE = 'lwgeom', data_dir)
}

CPL_linesubstring <- function(sfc, from, to, tolerance = 0.0) {
    .Call('_lwgeom_CPL_linesubstring', PACKAGE = 'lwgeom', sfc, from, to, tolerance)
}

#' create sfc object from tiny well-known binary (twkb)
#'
#' create sfc object from tiny well-known binary (twkb)
#' @param x list with raw vectors, of class \code{TWKB}
#' @param ... ignored
#' @seealso https://github.com/TWKB/Specification/blob/master/twkb.md
#' @examples
#' l = structure(list(as.raw(c(0x02, 0x00, 0x02, 0x02, 0x02, 0x08, 0x08))), class = "TWKB")
#' library(sf) # load generic
#' st_as_sfc(l)
#' @export
st_as_sfc.TWKB = function(x, ...) {
	st_sfc(CPL_sfc_from_twkb(x))
}

suppressPackageStartupMessages(library(lwgeom))
suppressPackageStartupMessages(library(sf))
nc = st_read(system.file("gpkg/nc.gpkg", package="sf"), quiet = TRUE)
nc = st_transform(nc, 3857)
st_perimeter_lwgeom(nc)[1:5]
st_perimeter_2d(nc)[1:5]

#' @importFrom Rcpp evalCpp
#' @import sf
#' @importFrom utils tail
#' @importFrom units set_units as_units
#' @useDynLib lwgeom
NULL

.onLoad = function(libname, pkgname) {
	# SET PROJ.db path:
	if (file.exists(system.file("proj/nad.lst", package = "sf")[1])) { # we are on Windows:
		prj = system.file("proj", package = "sf")[1]
		CPL_set_data_dir(prj) # if existing, uses C API to set path
		CPL_use_proj4_init_rules(1L)
	}
}

.onAttach = function(libname, pkgname) {
	esv = lwgeom_extSoftVersion()
	m = paste0("Linking to liblwgeom ", esv["lwgeom"],
		", GEOS ", esv["GEOS"],
		", PROJ ", esv["proj.4"])
	CPL_init_lwgeom(NA_character_)
	packageStartupMessage(m)
	sf = sf_extSoftVersion()
	if (sf["GEOS"] != esv["GEOS"])
		warning(paste("GEOS versions differ: lwgeom has", esv["GEOS"], "sf has", sf["GEOS"]))
	if (sf["proj.4"] != esv["proj.4"])
		warning(paste("PROJ versions differ: lwgeom has", esv["proj.4"], "sf has", sf["proj.4"]))

	
}

#' Provide the external dependencies versions of the libraries linked to sf
#' 
#' Provide the external dependencies versions of the libraries linked to sf
#' @export
lwgeom_extSoftVersion = function() {
	structure(c(CPL_lwgeom_version(), CPL_geos_version(), CPL_proj_version()),
		names = c("lwgeom", "GEOS", "proj.4"))
}

l = structure(list(as.raw(c(0x02, 0x00, 0x02, 0x02, 0x02, 0x08, 0x08))), class = "TWKB")
suppressPackageStartupMessages(library(sf))
suppressPackageStartupMessages(library(lwgeom))
st_as_sfc(l)
st_as_sfc(structure(list(l[[1]], l[[1]]), class = "TWKB"))

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bearing.R
\name{st_geod_azimuth}
\alias{st_geod_azimuth}
\title{compute azimuth between sequence of points}
\usage{
st_geod_azimuth(x, y = NULL)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{sfg}}

\item{y}{object of class \code{sf}, \code{sfc} or \code{sfg}, or NULL}
}
\description{
compute azimuth between sequence of points
}
\examples{
library(sf)
p = st_sfc(st_point(c(7,52)), st_point(c(8,53)), crs = 4326)
st_geod_azimuth(p)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/valid.R
\name{lwgeom_make_valid}
\alias{lwgeom_make_valid}
\title{Make an invalid geometry valid}
\usage{
lwgeom_make_valid(x)
}
\arguments{
\item{x}{object of class \code{sfc}}
}
\description{
Make an invalid geometry valid
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perimeter.R
\name{perimeter}
\alias{perimeter}
\alias{st_perimeter_lwgeom}
\alias{st_perimeter_2d}
\title{Compute perimeter from polygons or other geometries}
\usage{
st_perimeter_lwgeom(x)

st_perimeter_2d(x)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{sfg}}
}
\value{
numerical vector with perimeter for each feature (geometry), with unit of measure when possible
}
\description{
Compute perimeter from polygons or other geometries
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrap_x.R
\name{st_wrap_x}
\alias{st_wrap_x}
\title{Splits input geometries by a vertical line and moves components falling on
one side of that line by a fixed amount}
\usage{
st_wrap_x(x, wrap, move)
}
\arguments{
\item{x}{object with geometries to be split}

\item{wrap}{x value of split line}

\item{move}{amount by which geometries falling to the left of the line
should be translated to the right}
}
\value{
object of the same class as \code{x}
}
\description{
Splits input geometries by a vertical line and moves components falling on
one side of that line by a fixed amount
}
\examples{
library(sf)
demo(nc, ask = FALSE, echo = FALSE)
x = st_wrap_x(nc, -78, 10)
plot(x[1])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split.R
\name{st_split}
\alias{st_split}
\title{Return a collection of geometries resulting by splitting a geometry}
\usage{
st_split(x, y)
}
\arguments{
\item{x}{object with geometries to be splitted}

\item{y}{object split with (blade); if \code{y} contains more than one feature geometry, the geometries are \link[sf:geos_combine]{st_combine} 'd}
}
\value{
object of the same class as \code{x}
}
\description{
Return a collection of geometries resulting by splitting a geometry
}
\examples{
library(sf)
l = st_as_sfc('MULTILINESTRING((10 10, 190 190), (15 15, 30 30, 100 90))')
pt = st_sfc(st_point(c(30,30)))
st_split(l, pt)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subdivide.R
\name{st_subdivide}
\alias{st_subdivide}
\title{Return a collection of geometries resulting by subdividing a geometry}
\usage{
st_subdivide(x, max_vertices)
}
\arguments{
\item{x}{object with geometries to be subdivided}

\item{max_vertices}{integer; maximum size of the subgeometries (at least 8)}
}
\value{
object of the same class as \code{x}
}
\description{
Return a collection of geometries resulting by subdividing a geometry
}
\examples{
library(sf)
demo(nc, ask = FALSE, echo = FALSE)
x = st_subdivide(nc, 10)
plot(x[1])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perimeter.R
\name{perimeter-deprecated}
\alias{perimeter-deprecated}
\alias{st_perimeter}
\title{compute perimeter from polygons or other geometries}
\usage{
st_perimeter(x)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{sfg}}
}
\description{
Deprecated.
Use \code{sf::st_perimeter()} or \code{st_perimeter_lwgeom()} instead.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snap_to_grid.R
\name{st_snap_to_grid}
\alias{st_snap_to_grid}
\title{Snap geometries to a grid}
\usage{
st_snap_to_grid(x, size, origin)
}
\arguments{
\item{x}{object with geometries to be snapped}

\item{size}{numeric or (length) units object; grid cell size in x-, y- (and possibly z- and m-) directions}

\item{origin}{numeric; origin of the grid}
}
\value{
object of the same class as \code{x}
}
\description{
Snap geometries to a grid
}
\examples{
# obtain data
library(sf)
x = st_read(system.file("gpkg/nc.gpkg", package="sf"), quiet = TRUE)[1, ] |>
    st_geometry() |>
    st_transform(3395)

# snap to a grid of 5000 m
err = try(y <- st_snap_to_grid(x, 5000))

# plot data for visual comparison
if (!inherits(err, "try-error")) {
 opar = par(mfrow = c(1, 2))
 plot(x, main = "orginal data")
 plot(y, main = "snapped to 5000 m")
 par(opar)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wkt.R
\name{st_astext}
\alias{st_astext}
\alias{st_asewkt}
\title{Return Well-known Text representation of simple feature geometry}
\usage{
st_astext(x, digits = getOption("digits"), ..., EWKT = FALSE)

st_asewkt(x, digits = options("digits"))
}
\arguments{
\item{x}{object of class \code{sfg}, \code{sfc}, or \code{sf}}

\item{digits}{integer; number of decimal digits to print}

\item{...}{ignored}

\item{EWKT}{logical; use PostGIS Enhanced WKT (includes srid)}
}
\description{
Return Well-known Text representation of simple feature geometry or coordinate reference system
}
\details{
The returned WKT representation of simple feature geometry conforms to the
\href{https://www.ogc.org/standards/sfa/}{simple features access} specification and extensions (if \code{EWKT = TRUE}),
\href{http://postgis.net/docs/using_postgis_dbmanagement.html#EWKB_EWKT}{known as EWKT}, supported by
PostGIS and other simple features implementations for addition of SRID to a WKT string.

\code{st_asewkt()} returns the Well-Known Text (WKT) representation of
the geometry with SRID meta data.
}
\examples{
library(sf)
pt <- st_sfc(st_point(c(1.0002,2.3030303)), crs = 4326)
st_astext(pt, 3)
st_asewkt(pt, 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clockwise.R
\name{st_is_polygon_cw}
\alias{st_is_polygon_cw}
\title{Check if a POLYGON or MULTIPOLYGON is clockwise}
\usage{
st_is_polygon_cw(x)
}
\arguments{
\item{x}{object with polygon geometries}
}
\value{
logical with length the same number of features in `x`
}
\description{
Check if a POLYGON or MULTIPOLYGON is clockwise. According to the 
'Right-hand-rule', outer rings should be clockwise, and inner holes
should be counter-clockwise
}
\examples{
library(sf)
polys <- st_sf(cw = c(FALSE, TRUE), 
               st_as_sfc(c('POLYGON ((0 0, 1 0, 1 1, 0 0))', 
                           'POLYGON ((1 1, 2 2, 2 1, 1 1))')))

st_is_polygon_cw(polys)
st_is_polygon_cw(st_geometry(polys))
st_is_polygon_cw(st_geometry(polys)[[1]])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geod.R
\name{geod}
\alias{geod}
\alias{st_geod_area}
\alias{st_geod_length}
\alias{st_geod_segmentize}
\alias{st_geod_covers}
\alias{st_geod_covered_by}
\alias{st_geod_distance}
\title{liblwgeom geodetic functions}
\usage{
st_geod_area(x)

st_geod_length(x)

st_geod_segmentize(x, max_seg_length)

st_geod_covers(x, y, sparse = TRUE)

st_geod_covered_by(x, y, sparse = TRUE)

st_geod_distance(x, y, tolerance = 0, sparse = FALSE)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{sfg}}

\item{max_seg_length}{segment length in degree, radians, or as a length unit (e.g., m)}

\item{y}{object of class \code{sf}, \code{sfc} or \code{sfg}}

\item{sparse}{logical; if \code{TRUE}, return a sparse matrix (object of class \code{sgbp}), otherwise, return a dense logical matrix.}

\item{tolerance}{double or length \code{units} value: if positive, the first distance less than \code{tolerance} is returned, rather than the true distance}
}
\description{
liblwgeom geodetic functions for length, area, segmentizing, covers
}
\details{
\code{st_area} will give an error message when the area spans the equator and \code{lwgeom} is linked to a proj.4 version older than 4.9.0 (see \link{lwgeom_extSoftVersion})

longitude coordinates returned are rescaled to [-180,180)
}
\note{
this function should is used by \link[sf:geos_measures]{st_distance}, do not use it directly
}
\examples{
library(sf)
nc = st_read(system.file("gpkg/nc.gpkg", package="sf"))
st_geod_area(nc[1:3,])
# st_area(nc[1:3,])
l = st_sfc(st_linestring(rbind(c(7,52), c(8,53))), crs = 4326)
st_geod_length(l)
library(units)
pol = st_polygon(list(rbind(c(0,0), c(0,60), c(60,60), c(0,0))))
x = st_sfc(pol, crs = 4326)
seg = st_geod_segmentize(x[1], set_units(10, km))
plot(seg, graticule = TRUE, axes = TRUE)
pole = st_polygon(list(rbind(c(0,80), c(120,80), c(240,80), c(0,80))))
pt = st_point(c(0,90))
x = st_sfc(pole, pt, crs = 4326)
st_geod_covers(x[c(1,1,1)], x[c(2,2,2,2)])
pole = st_polygon(list(rbind(c(0,80), c(120,80), c(240,80), c(0,80))))
pt = st_point(c(30,70))
x = st_sfc(pole, pt, crs = 4326)
st_geod_distance(x, x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twkb.R
\name{st_as_sfc.TWKB}
\alias{st_as_sfc.TWKB}
\title{create sfc object from tiny well-known binary (twkb)}
\usage{
\method{st_as_sfc}{TWKB}(x, ...)
}
\arguments{
\item{x}{list with raw vectors, of class \code{TWKB}}

\item{...}{ignored}
}
\description{
create sfc object from tiny well-known binary (twkb)
}
\examples{
l = structure(list(as.raw(c(0x02, 0x00, 0x02, 0x02, 0x02, 0x08, 0x08))), class = "TWKB")
library(sf) # load generic
st_as_sfc(l)
}
\seealso{
https://github.com/TWKB/Specification/blob/master/twkb.md
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split.R
\name{st_linesubstring}
\alias{st_linesubstring}
\title{get substring from linestring}
\usage{
st_linesubstring(x, from, to, tolerance, ...)
}
\arguments{
\item{x}{object of class \code{sfc}, \code{sf} or \code{sfg}}

\item{from}{relative distance from origin (in [0,1])}

\item{to}{relative distance from origin (in [0,1])}

\item{tolerance}{tolerance parameter, when to snap to line node node}

\item{...}{ignored}
}
\value{
object of class \code{sfc}
}
\description{
get substring from linestring
}
\examples{
library(sf)
lines = st_sfc(st_linestring(rbind(c(0,0), c(1,2), c(2,0))), crs = 4326)
spl = st_linesubstring(lines, 0.2, 0.8) # should warn
plot(st_geometry(lines), col = 'red', lwd = 3)
plot(spl, col = 'black', lwd = 3, add = TRUE)
st_linesubstring(lines, 0.49999, 0.8) # three points
st_linesubstring(lines, 0.49999, 0.8, 0.001) # two points: snap start to second node
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clockwise.R
\name{st_force_polygon_cw}
\alias{st_force_polygon_cw}
\title{Force a POLYGON or MULTIPOLYGON to be clockwise}
\usage{
st_force_polygon_cw(x)
}
\arguments{
\item{x}{object with polygon geometries}
}
\value{
object of the same class as \code{x}
}
\description{
Check if a POLYGON or MULTIPOLYGON is clockwise, and if not make it so. 
According to the 'Right-hand-rule', outer rings should be clockwise, and 
inner holes should be counter-clockwise
}
\examples{
library(sf)
polys <- st_sf(cw = c(FALSE, TRUE), 
               st_as_sfc(c('POLYGON ((0 0, 1 0, 1 1, 0 0))', 
                           'POLYGON ((1 1, 2 2, 2 1, 1 1))')))

st_force_polygon_cw(polys)
st_force_polygon_cw(st_geometry(polys))
st_force_polygon_cw(st_geometry(polys)[[1]])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform.R
\name{st_transform_proj}
\alias{st_transform_proj}
\alias{st_transform_proj.sfc}
\alias{st_transform_proj.sf}
\alias{st_transform_proj.sfg}
\title{Transform or convert coordinates of simple features directly with Proj.4 (bypassing GDAL)}
\usage{
st_transform_proj(x, crs, ...)

\method{st_transform_proj}{sfc}(x, crs, ...)

\method{st_transform_proj}{sf}(x, crs, ...)

\method{st_transform_proj}{sfg}(x, crs, ...)
}
\arguments{
\item{x}{object of class sf, sfc or sfg}

\item{crs}{character; target CRS, or, in case of a length 2 character vector, source and target CRS}

\item{...}{ignored}
}
\description{
Transform or convert coordinates of simple features directly with Proj.4 (bypassing GDAL)
}
\details{
Transforms coordinates of object to new projection, using PROJ directly rather than the GDAL API used by \link[sf]{st_transform}.

If \code{crs} is a single CRS, it forms the target CRS, and in that case the source CRS is obtained as \code{st_crs(x)}. Since this presumes that the source CRS is accepted by GDAL (which is not always the case), a second option is to specify the source and target CRS as two proj4strings in argument \code{crs}. In the latter case, \code{st_crs(x)} is ignored and may well be \code{NA}.

The \code{st_transform_proj} method for \code{sfg} objects assumes that the CRS of the object is available as an attribute of that name.
}
\examples{
library(sf)
p1 = st_point(c(7,52))
p2 = st_point(c(-30,20))
sfc = st_sfc(p1, p2, crs = 4326)
sfc
st_transform_proj(sfc, "+proj=wintri")
library(sf)
nc = st_read(system.file("shape/nc.shp", package="sf"))
st_transform_proj(nc[1,], "+proj=wintri +over")
st_transform_proj(structure(p1, proj4string = "+init=epsg:4326"), "+init=epsg:3857")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bounding_circle.R
\name{bounding_circle}
\alias{bounding_circle}
\alias{st_minimum_bounding_circle}
\alias{st_minimum_bounding_radius}
\title{Generate the minimum bounding circle}
\usage{
st_minimum_bounding_circle(x, nQuadSegs = 30)

st_minimum_bounding_radius(x)
}
\arguments{
\item{x}{object of class \code{sfg}, \code{sfc} or \code{sf}}

\item{nQuadSegs}{number of segments per quadrant (passed to \code{st_buffer})}
}
\value{
Object of the same class as \code{x}
}
\description{
Generate the minimum bounding circle
}
\details{
\code{st_minimum_bounding_circle} uses the \code{lwgeom_calculate_mbc} method also used by the PostGIS command \code{ST_MinimumBoundingCircle}. \code{st_minimum_bounding_radius} also uses the \code{lwgeom_calculate_mbc} method, but returns the centers (as a geometry) and the respective radii (as a column).
}
\examples{
library(sf)

x = st_multipoint(matrix(c(0,1,0,1),2,2))
y = st_multipoint(matrix(c(0,0,1,0,1,1),3,2))

mbcx = st_minimum_bounding_circle(x)
mbcy = st_minimum_bounding_circle(y)

if (.Platform$OS.type != "windows") {
  plot(mbcx, axes=TRUE); plot(x, add=TRUE)
  plot(mbcy, axes=TRUE); plot(y, add=TRUE)
}

nc = st_read(system.file("gpkg/nc.gpkg", package="sf"))
state = st_union(st_geometry(nc))

if (.Platform$OS.type != "windows") {
  plot(st_minimum_bounding_circle(state), asp=1)
  plot(state, add=TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/startpoint.R
\name{st_startpoint}
\alias{st_startpoint}
\alias{st_endpoint}
\title{Return the start and end points from lines}
\usage{
st_startpoint(x)

st_endpoint(x)
}
\arguments{
\item{x}{line of class \code{sf}, \code{sfc} or \code{sfg}}
}
\value{
\code{sf} object representing start and end points
}
\description{
Return the start and end points from lines
}
\details{
see \url{https://postgis.net/docs/ST_StartPoint.html} and \url{https://postgis.net/docs/ST_EndPoint.html}.
}
\examples{
library(sf)
m = matrix(c(0, 1, 2, 0, 1, 4), ncol = 2)
l = st_sfc(st_linestring(m))
lwgeom::st_startpoint(l)
lwgeom::st_endpoint(l)
l2 = st_sfc(st_linestring(m), st_linestring(m[3:1, ]))
lwgeom::st_startpoint(l2)
lwgeom::st_endpoint(l2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geohash.R
\name{st_geohash}
\alias{st_geohash}
\alias{st_geom_from_geohash}
\title{compute geohash from (average) coordinates}
\usage{
st_geohash(x, precision = 0)

st_geom_from_geohash(
  hash,
  precision = -1,
  crs = st_crs("OGC:CRS84"),
  raw = FALSE
)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{sfg}}

\item{precision}{integer; precision (length) of geohash returned. From the liblwgeom source: ``where the precision is non-positive, a precision based on the bounds of the feature. Big features have loose precision. Small features have tight precision.''}

\item{hash}{character vector with geohashes}

\item{crs}{object of class `crs`}

\item{raw}{logical; if `TRUE`, return a matrix with bounding box coordinates on each row}
}
\value{
`st_geohash` returns a character vector with geohashes; empty or full geometries result in `NA`

`st_geom_from_geohash` returns a (bounding box) polygon for each geohash if `raw` is `FALSE`, if `raw` is `TRUE` a matrix is returned with bounding box coordinates on each row.
}
\description{
compute geohash from (average) coordinates
}
\details{
see \url{https://en.wikipedia.org/wiki/Geohash}.
}
\examples{
library(sf)
lwgeom::st_geohash(st_sfc(st_point(c(1.5,3.5)), st_point(c(0,90))), 2)
lwgeom::st_geohash(st_sfc(st_point(c(1.5,3.5)), st_point(c(0,90))), 10)
st_geom_from_geohash(c('9qqj7nmxncgyy4d0dbxqz0', 'up'), raw = TRUE)
o = options(digits = 20) # for printing purposes
st_geom_from_geohash(c('9qqj7nmxncgyy4d0dbxqz0', 'u1hzz631zyd63zwsd7zt'))
st_geom_from_geohash('9qqj7nmxncgyy4d0dbxqz0', 4) 
st_geom_from_geohash('9qqj7nmxncgyy4d0dbxqz0', 10)
options(o)
}
